/*
 * Copyright (c) 2015, AIOI・SYSTEMS CO., LTD.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */

package com.aioisystems.smarttagsample;

import com.aioisystems.imaging.DisplayPainter;
import com.aioisystems.smarttag.ProgressListener;
import com.aioisystems.smarttag.TagActionListener;
import com.aioisystems.smarttagsample.CameraActivity;
import com.aioisystems.smarttagsample.Common;
import com.aioisystems.smarttagsample.EditActivity;
import com.aioisystems.smarttagsample.MainActivity;
import com.aioisystems.smarttagsample.SmartTag;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.net.Uri;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.NfcF;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;

public class MainActivity extends Activity {
	private NfcAdapter mAdapter = null;
	private PendingIntent mPendingIntent;
	private IntentFilter[] mFilters;
	private String[][] mTechLists;
	private int mNfcStatus;

	private SmartTagTask mTagTask;

	private SmartTag mSmartTag;
	private volatile boolean mIsBusy = false;

	public static final int SHOW_EDIT = 1;
	public static final int SHOW_CAMERA_PREVIEW = 2;
	public static final int SHOW_URL_EDIT = 3;

	private static final int INIT_NFC_OK = 0;
	private static final int INIT_NFC_NOTFOUND = 1;
	private static final int INIT_NFC_DISABLED = 2;
	private static final int INIT_NFC_ERROR = 9;

	private Handler mHandler = new Handler();

	private TextView mErrorView;
	private TextView mIdView;
	private TextView mUserTextView;
	private LinearLayout mCameraItemLayout;
	private ImageView mCameraPreview;
	private LinearLayout mSaveImageLayout;
	private Spinner mSaveNoList;
	private Spinner mLayoutList;
	private LinearLayout mSelectImageLayout;
	private int mLayoutNo = 1;
	private LinearLayout mWriteDataLayout;
	private TextView mUrlView;
	private TextView mCaution1;
	private ProgressBar mProgress;

	private boolean mIsEdited = false;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mErrorView = (TextView)findViewById(R.id.errorView);
        mIdView = (TextView)findViewById(R.id.idmView);
        mIdView.setText("ID: ----");

        mSmartTag = new SmartTag();
        mSmartTag.setProgressListener(mProgressListener);
        mSmartTag.setTagActionListener(mTagActionListener);
        mSmartTag.setFunctionNo(SmartTag.FN_SHOW_STATUS);

        mUserTextView = (TextView)findViewById(R.id.userTextView);
        mUserTextView.setText(getString(R.string.msgTouchToEdit));
        mUserTextView.setVisibility(View.GONE);

        mUserTextView.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				showEditActivity(v);
			}
		});

        mWriteDataLayout = (LinearLayout)findViewById(R.id.writeDataLayout);
        mWriteDataLayout.setVisibility(View.GONE);
        mUrlView = (TextView)findViewById(R.id.urlTextView);
        mUrlView.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				showUrlEditActivity(v);

			}
		});
        mCaution1 = (TextView)findViewById(R.id.caution1);
        mCaution1.setVisibility(View.GONE);

        //camera
        mCameraPreview = (ImageView)findViewById(R.id.cameraPreview);
    	mCameraItemLayout = (LinearLayout)findViewById(R.id.cameraItemLayout);
    	mCameraItemLayout.setVisibility(View.GONE);

    	ImageButton button = (ImageButton)findViewById(R.id.cameraButton);
    	button.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent intent = new Intent(getApplicationContext(), CameraActivity.class);
				startActivityForResult(intent, SHOW_CAMERA_PREVIEW);
			}
		});

    	//saveing layout
    	mSaveImageLayout = (LinearLayout)findViewById(R.id.saveImageLayout);
    	mSaveImageLayout.setVisibility(View.GONE);
    	mSaveNoList = (Spinner)findViewById(R.id.saveNoList);

    	//layout numbers
    	String[] items = new String[]{"1", "2", "3", "4", "5", "6",
    			"7", "8", "9", "10", "11", "12",};
    	ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, items);
    	mSaveNoList.setAdapter(adapter);
    	adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	mSaveNoList.setPrompt(getString(R.string.layoutSelectionPrompt));

    	//loading layout
    	mSelectImageLayout = (LinearLayout)findViewById(R.id.selectImageLayout);
    	mSelectImageLayout.setVisibility(View.GONE);
    	mLayoutList = (Spinner)findViewById(R.id.layoutNoList);
    	adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, items);
    	mLayoutList.setAdapter(adapter);
    	adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	mLayoutList.setPrompt(getString(R.string.layoutSelectionPrompt));

    	RadioGroup group = (RadioGroup)findViewById(R.id.radioGroup1);
    	group.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(RadioGroup group, int checkedId) {
				onRadioChanged(checkedId);
			}
		});

    	mProgress = (ProgressBar)findViewById(R.id.progressBar1);
    	mProgress.setVisibility(View.INVISIBLE);

    	//Prepares NFC
        mNfcStatus = startNfc();
        switch(mNfcStatus){
        case INIT_NFC_OK:
        	mErrorView.setText(getString(R.string.scanTagPrompt));
			break;
        case INIT_NFC_NOTFOUND:
        	mErrorView.setText(getString(R.string.msgCannotStart));
			break;
        case INIT_NFC_DISABLED:
        	mErrorView.setText(getString(R.string.msgNfcDisabled));
        	break;
        case INIT_NFC_ERROR:
        	mErrorView.setText(getString(R.string.msgCannotStart));
        	break;
		}
    }

    @Override
    protected void onResume() {
    	super.onResume();

    	Common.addLogi("start adapter");
    	if(mNfcStatus == INIT_NFC_OK){
    		mAdapter.enableForegroundDispatch(
    				this, mPendingIntent, mFilters, mTechLists);
    	}
    }

    @Override
    protected void onPause() {
    	super.onPause();

		Common.addLogi("stop adapter");
    	try{
    		if(mNfcStatus == INIT_NFC_OK){
    			mAdapter.disableForegroundDispatch(this);
    		}
    	}catch(Exception e){
    		Common.addLoge(e.getMessage());
    	}
    }

    @Override
	protected void onNewIntent(Intent intent){
    	Common.addLogi("onNewIntent");

    	if(mIsBusy){
			return;
		}

		//IDm
    	Tag tag = (Tag)intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
    	byte[] idm = intent.getByteArrayExtra(NfcAdapter.EXTRA_ID);
    	mIdView.setText("ID: " + Common.makeHexText(idm));

		mSmartTag.selectTarget(idm, tag);

		if(!mSmartTag.isSmartTag()){
			mErrorView.setText(getString(R.string.scanTagPrompt));
			return;
		}

        mTagTask = new SmartTagTask();

		setParameter();
		mIsBusy = true;
		mTagTask.execute();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	if(requestCode == SHOW_EDIT){
    		//Result of editing text.
    		if(data != null){
	    		String text = data.getCharSequenceExtra("TEXT").toString();
	    		if(text.equals("")){
	    			mIsEdited = false;
	    			text = getString(R.string.msgTouchToEdit);
	    			mSmartTag.setDrawText(" ");
	    		}else{
	    			mIsEdited = true;
	    			mSmartTag.setDrawText(text);
	    		}
	    		mUserTextView.setText(text);
    		}

    	}else if(requestCode == SHOW_CAMERA_PREVIEW){
    		//Result of camera activity.
    		if(resultCode == RESULT_OK){
    			Bitmap bitmap = (Bitmap)data.getParcelableExtra("BITMAP");

				DisplayPainter painter = new DisplayPainter();
				painter.putImage(bitmap, 0, 0, true);
				bitmap.recycle();

				bitmap = painter.getPreviewImage();
				mSmartTag.setCameraImage(bitmap);

				mCameraPreview.setVisibility(View.VISIBLE);
				mCameraPreview.setImageBitmap(bitmap);
			}
    	}else if(requestCode == SHOW_URL_EDIT){
    		//Result of editing URL
    		if(data != null){
	    		String text = data.getCharSequenceExtra("TEXT").toString().trim();
	    		if(text.equals("")){
	    			text = "http://";
	    			mSmartTag.setWriteText("");
	    		}else{
	    			mSmartTag.setWriteText(text);
	    		}
	    		mUrlView.setText(text);
    		}
    	}
    }

    /**
     * Starts NFC
     * @return 0:success 1:adapter not found 2:NFC disabled
     */
    private int startNfc(){
        try{
        	mAdapter = NfcAdapter.getDefaultAdapter(this);

        	if(mAdapter == null){
	        	return INIT_NFC_NOTFOUND;
	        }else{
		        if(!mAdapter.isEnabled()){
		        	return INIT_NFC_DISABLED;
		        }
	        	mPendingIntent = PendingIntent.getActivity(this, 0,
		        		new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);

		        mFilters = new IntentFilter[]{
		        		new IntentFilter(NfcAdapter.ACTION_TECH_DISCOVERED),
		        	};
		        mTechLists = new String[][]{new String[] { NfcF.class.getName() }};
	        }
        }catch(Exception e){
        	Common.addLoge(e.toString());
        	return INIT_NFC_ERROR;
        }
        return INIT_NFC_OK;
    }

    private void onRadioChanged(int checkedId){

    	mUserTextView.setVisibility(View.GONE);
    	mCameraItemLayout.setVisibility(View.GONE);
    	mSaveImageLayout.setVisibility(View.GONE);
    	mSelectImageLayout.setVisibility(View.GONE);
    	mWriteDataLayout.setVisibility(View.GONE);
    	mCaution1.setVisibility(View.GONE);

    	if(checkedId == R.id.radioShowStatus){
    		mSmartTag.setFunctionNo(SmartTag.FN_SHOW_STATUS);
    	}else if(checkedId == R.id.radioShowStatus27){
    		mSmartTag.setFunctionNo(SmartTag.FN_SHOW_STATUS27);
    	}else if(checkedId == R.id.radioShowStatus29){
    		mSmartTag.setFunctionNo(SmartTag.FN_SHOW_STATUS29);
    	}else if(checkedId == R.id.radioShowUserImage){
			mSelectImageLayout.setVisibility(View.VISIBLE);
			mSmartTag.setFunctionNo(SmartTag.FN_CHANGE_LAYOUT);
		}else if(checkedId == R.id.radioShowCameraImage){
			mCameraItemLayout.setVisibility(View.VISIBLE);
			mSmartTag.setFunctionNo(SmartTag.FN_DRAW_CAMERA_IMAGE);
		}else if(checkedId == R.id.radioDrawText){
			mUserTextView.setVisibility(View.VISIBLE);
			mSmartTag.setFunctionNo(SmartTag.FN_DRAW_TEXT);
		}else if(checkedId == R.id.radioSaveImage){
    		mSaveImageLayout.setVisibility(View.VISIBLE);
    		mSmartTag.setFunctionNo(SmartTag.FN_SAVE_LAYOUT);
		}else if(checkedId == R.id.radioWriteData){
			mWriteDataLayout.setVisibility(View.VISIBLE);
			mSmartTag.setFunctionNo(SmartTag.FN_WRITE_DATA);
		}else if(checkedId == R.id.radioReadData){
			mCaution1.setVisibility(View.VISIBLE);
			mSmartTag.setFunctionNo(SmartTag.FN_READ_DATA);
		}else if(checkedId == R.id.radioClear){
			mSmartTag.setFunctionNo(SmartTag.FN_CLEAR_DISPLAY);
		}
	}

    /**
     * Sets a function number and parameters.
     */
    private void setParameter()
    {
    	switch(mSmartTag.getFunctionNo()){
    	case SmartTag.FN_SAVE_LAYOUT:
			String item = (String)mSaveNoList.getSelectedItem();
        	mSmartTag.setSaveNo(Integer.parseInt(item));
    		break;
    	case SmartTag.FN_CHANGE_LAYOUT:
    		String numberToChange = (String)mLayoutList.getSelectedItem();
    		mSmartTag.setLayoutNo(Integer.parseInt(numberToChange));
    		break;
    	case SmartTag.FN_WRITE_DATA:
    		mSmartTag.setWriteText(mUrlView.getText().toString());
    		break;
    	}
    }

    /**
     * Shows the text editing activity.
     * @param v
     */
    private void showEditActivity(View v){
    	Intent intent = new Intent(this, EditActivity.class);
    	String text = "";
    	if(mIsEdited){
    		text = mUserTextView.getText().toString();
    	}
    	intent.putExtra("TEXT", text);
    	intent.putExtra("REQ_CODE", SHOW_EDIT);
    	startActivityForResult(intent, SHOW_EDIT);
    }

    /**
     * Shows the URL editing activity.
     * @param v
     */
    private void showUrlEditActivity(View v){
    	Intent intent = new Intent(this, EditActivity.class);
    	String text =  mUrlView.getText().toString();

    	intent.putExtra("TEXT", text);
    	intent.putExtra("REQ_CODE", SHOW_URL_EDIT);
    	startActivityForResult(intent, SHOW_URL_EDIT);
    }

    /**
     * Opens the URL.
     * @param url
     */
    private void openUrl(String url){
    	try{
	    	if(url.startsWith("http://")
	    			|| url.startsWith("https://")){
		    	Intent intent =
		    			new Intent(Intent.ACTION_VIEW, Uri.parse(url));
		    	startActivity(intent);
	    	}
    	}catch(Exception err){
    		Common.addLoge(err.toString());
    	}
    }

    /**
     * Implements the async-task of the smart-tag.
     */
    private class SmartTagTask extends AsyncTask<Void, Void, Void> {

    	@Override
    	protected void onPreExecute() {
    		mProgress.setVisibility(View.VISIBLE);
    		mProgress.setIndeterminate(true);

    		mErrorView.setText(getString(R.string.msgProcessing));
    	}

    	@Override
    	protected Void doInBackground(Void... params) {

    		mSmartTag.startSession();
    		return null;
    	}

    	@Override
    	protected void onProgressUpdate(Void... values) {
    		super.onProgressUpdate(values);
    	}

    	@Override
    	protected void onPostExecute(Void result) {

    		mProgress.setVisibility(View.INVISIBLE);

    		Exception error = mSmartTag.getLastError();
    		if(error != null){
    			mErrorView.setText(getString(R.string.msgIoError));
    		}else{

	    		int function = mSmartTag.getFunctionNo();
				if(function == SmartTag.FN_CHANGE_LAYOUT){
					mLayoutNo++;
	    			if(mLayoutNo > 12){
	    				mLayoutNo = 1;
	    			}
	    			mLayoutList.setSelection(mLayoutNo - 1);
				}else if(function == SmartTag.FN_READ_DATA){
					openUrl(mSmartTag.getReadText());
				}

	    		mErrorView.setText(getString(R.string.msgDone));
    		}
    		mIsBusy = false;

    	}
    }


    private ProgressListener mProgressListener = new ProgressListener() {

		@Override
		public void onProgress(final int progress) {
			mHandler.post(new Runnable() {
				public void run() {
					mProgress.setProgress(progress);
				};
			});
		}

		@Override
		public void onMaxChanged(final int maximum) {
			mHandler.post(new Runnable() {
				public void run() {
					mProgress.setIndeterminate(false);
					mProgress.setMax(maximum);
				};
			});
		}
	};

    private TagActionListener mTagActionListener = new TagActionListener() {

		@Override
		public void onReleased() {
			mHandler.post(new Runnable() {
				public void run() {
					mErrorView.setText(getString(R.string.scanTagPrompt));
				}
			});
		}
	};
}